@@ -17,7 +17,7 @@
 use crate::{impl_bytes, BTreeMap, *};
 use core::ops::{Add, AddAssign};
 use snarkvm_curves::traits::{AffineCurve, PairingCurve, PairingEngine, ProjectiveCurve};
-use snarkvm_fields::{PrimeField, Zero};
+use snarkvm_fields::{ConstraintFieldError, PrimeField, ToConstraintField, Zero};
 use snarkvm_utilities::{
     bytes::ToBytes,
     error,
@@ -96,6 +96,56 @@ pub struct VerifierKey<E: PairingEngine> {
 }
 impl_bytes!(VerifierKey);
 
+impl<E: PairingEngine> ToConstraintField<E::Fq> for VerifierKey<E>
+where
+    E::G1Affine: ToConstraintField<E::Fq>,
+    E::G2Affine: ToConstraintField<E::Fq>,
+{
+    fn to_field_elements(&self) -> Result<Vec<E::Fq>, ConstraintFieldError> {
+        let mut res = Vec::new();
+
+        res.extend_from_slice(&self.g.to_field_elements().unwrap());
+        res.extend_from_slice(&self.gamma_g.to_field_elements().unwrap());
+        res.extend_from_slice(&self.h.to_field_elements().unwrap());
+        res.extend_from_slice(&self.beta_h.to_field_elements().unwrap());
+
+        Ok(res)
+    }
+}
+
+/// `PreparedVerifierKey` is the fully prepared version for checking evaluation proofs for a given commitment.
+/// We omit gamma here for simplicity.
+#[derive(Derivative)]
+#[derivative(Default(bound = ""), Clone(bound = ""), Debug(bound = ""))]
+pub struct PreparedVerifierKey<E: PairingEngine> {
+    /// The generator of G1, prepared for power series.
+    pub prepared_g: Vec<E::G1Affine>,
+    /// The generator of G2, prepared for use in pairings.
+    pub prepared_h: <E::G2Affine as PairingCurve>::Prepared,
+    /// \beta times the above generator of G2, prepared for use in pairings.
+    pub prepared_beta_h: <E::G2Affine as PairingCurve>::Prepared,
+}
+
+impl<E: PairingEngine> PreparedVerifierKey<E> {
+    /// prepare `PreparedVerifierKey` from `VerifierKey`
+    pub fn prepare(vk: &VerifierKey<E>) -> Self {
+        let supported_bits = E::Fr::size_in_bits();
+
+        let mut prepared_g = Vec::<E::G1Affine>::new();
+        let mut g = E::G1Projective::from(vk.g.clone());
+        for _ in 0..supported_bits {
+            prepared_g.push(g.clone().into());
+            g.double_in_place();
+        }
+
+        Self {
+            prepared_g,
+            prepared_h: vk.prepared_h.clone(),
+            prepared_beta_h: vk.prepared_beta_h.clone(),
+        }
+    }
+}
+
 /// `Commitment` commits to a polynomial. It is output by `KZG10::commit`.
 #[derive(Derivative)]
 #[derivative(
@@ -139,6 +189,47 @@ impl<'a, E: PairingEngine> AddAssign<(E::Fr, &'a Commitment<E>)> for Commitment<
     }
 }
 
+impl<E: PairingEngine> ToConstraintField<E::Fq> for Commitment<E>
+where
+    E::G1Affine: ToConstraintField<E::Fq>,
+{
+    fn to_field_elements(&self) -> Result<Vec<E::Fq>, ConstraintFieldError> {
+        self.0.to_field_elements()
+    }
+}
+
+/// `PreparedCommitment` commits to a polynomial and prepares for mul_bits.
+#[derive(Derivative)]
+#[derivative(
+    Default(bound = ""),
+    Hash(bound = ""),
+    Clone(bound = ""),
+    Debug(bound = ""),
+    PartialEq(bound = ""),
+    Eq(bound = "")
+)]
+pub struct PreparedCommitment<E: PairingEngine>(
+    /// The commitment is a group element.
+    pub Vec<E::G1Affine>,
+);
+
+impl<E: PairingEngine> PreparedCommitment<E> {
+    /// prepare `PreparedCommitment` from `Commitment`
+    pub fn prepare(comm: &Commitment<E>) -> Self {
+        let mut prepared_comm = Vec::<E::G1Affine>::new();
+        let mut cur = E::G1Projective::from(comm.0.clone());
+
+        let supported_bits = E::Fr::size_in_bits();
+
+        for _ in 0..supported_bits {
+            prepared_comm.push(cur.clone().into());
+            cur.double_in_place();
+        }
+
+        Self { 0: prepared_comm }
+    }
+}
+
 /// `Randomness` hides the polynomial inside a commitment. It is output by `KZG10::commit`.
 #[derive(Derivative)]
 #[derivative(
